@@ -200,7 +200,7 @@ static int fuse_mount_opt_proc(void *data, const char *arg, int key,
 		return fuse_opt_add_opt(&mo->kernel_opts, arg);
 
 	case KEY_FUSERMOUNT_OPT:
-		return fuse_opt_add_opt(&mo->fusermount_opts, arg);
+		return fuse_opt_add_opt_escaped(&mo->fusermount_opts, arg);
 
 	case KEY_SUBTYPE_OPT:
 		return fuse_opt_add_opt(&mo->subtype_opt, arg);
@@ -484,6 +484,7 @@ static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 		goto out_close;
 	}
 
+#ifndef IGNORE_MTAB
 	if (geteuid() == 0) {
 		char *newmnt = fuse_mnt_resolve_path("fuse", mnt);
 		res = -1;
@@ -496,6 +497,7 @@ static int fuse_mount_sys(const char *mnt, struct mount_opts *mo,
 		if (res == -1)
 			goto out_umount;
 	}
+#endif /* IGNORE_MTAB */
 	free(type);
 	free(source);
 
