@@ -407,8 +407,15 @@ static int unmount_fuse_locked(const char *mnt, int quiet, int lazy)
 	if (res == -1)
 		goto out;
 
-	res = fuse_mnt_umount(progname, mnt, last, lazy);
+	res = umount2(last, lazy ? 2 : 0);
+	if (res == -1 && !quiet) {
+		fprintf(stderr,
+			"%s: failed to unmount %s: %s\n",
+			progname, mnt, strerror(errno));
+	}
 
+	if (res == 0)
+		res = fuse_mnt_remove_mount(progname, mnt);
 out:
 	free(copy);
 	if (currdir_fd != -1) {
@@ -649,7 +656,9 @@ static int opt_eq(const char *s, unsigned len, const char *opt)
 static int get_string_opt(const char *s, unsigned len, const char *opt,
 			  char **val)
 {
+	int i;
 	unsigned opt_len = strlen(opt);
+	char *d;
 
 	if (*val)
 		free(*val);
@@ -659,8 +668,15 @@ static int get_string_opt(const char *s, unsigned len, const char *opt,
 		return 0;
 	}
 
-	memcpy(*val, s + opt_len, len - opt_len);
-	(*val)[len - opt_len] = '\0';
+	d = *val;
+	s += opt_len;
+	len -= opt_len;
+	for (i = 0; i < len; i++) {
+		if (s[i] == '\\' && i + 1 < len)
+			i++;
+		*d++ = s[i];
+	}
+	*d = '\0';
 	return 1;
 }
 
@@ -691,7 +707,12 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 		unsigned len;
 		const char *fsname_str = "fsname=";
 		const char *subtype_str = "subtype=";
-		for (len = 0; s[len] && s[len] != ','; len++);
+		for (len = 0; s[len]; len++) {
+			if (s[len] == '\\' && s[len + 1])
+				len++;
+			else if (s[len] == ',')
+				break;
+		}
 		if (begins_with(s, fsname_str)) {
 			if (!get_string_opt(s, len, fsname_str, &fsname))
 				goto err;
@@ -861,7 +882,7 @@ static int check_version(const char *dev)
 }
 
 static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
-		      int *mountpoint_fd)
+		      int *mountpoint_fd, int *isdir)
 {
 	int res;
 	const char *mnt = *mntp;
@@ -879,6 +900,7 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
 		return 0;
 
 	if (S_ISDIR(stbuf->st_mode)) {
+		*isdir = 1;
 		*currdir_fd = open(".", O_RDONLY);
 		if (*currdir_fd == -1) {
 			fprintf(stderr,
@@ -886,10 +908,16 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
 				progname, strerror(errno));
 			return -1;
 		}
-		res = chdir(mnt);
+		*mountpoint_fd = open(mnt, O_RDONLY);
+		if (*mountpoint_fd == -1) {
+			fprintf(stderr, "%s: failed to open %s: %s\n",
+				progname, mnt, strerror(errno));
+			return -1;
+		}
+		res = fchdir(*mountpoint_fd);
 		if (res == -1) {
 			fprintf(stderr,
-				"%s: failed to chdir to mountpoint: %s\n",
+				"%s: failed to fchdir to mountpoint: %s\n",
 				progname, strerror(errno));
 			return -1;
 		}
@@ -1015,6 +1043,7 @@ static int mount_fuse(const char *mnt, const char *opts)
 	const char *real_mnt = mnt;
 	int currdir_fd = -1;
 	int mountpoint_fd = -1;
+	int isdir = 0;
 
 	fd = open_fuse_device(&dev);
 	if (fd == -1)
@@ -1035,7 +1064,7 @@ static int mount_fuse(const char *mnt, const char *opts)
 	res = check_version(dev);
 	if (res != -1) {
 		res = check_perm(&real_mnt, &stbuf, &currdir_fd,
-				 &mountpoint_fd);
+				 &mountpoint_fd, &isdir);
 		restore_privs();
 		if (res != -1)
 			res = do_mount(real_mnt, &type, stbuf.st_mode & S_IFMT,
@@ -1049,22 +1078,37 @@ static int mount_fuse(const char *mnt, const char *opts)
 		close(currdir_fd);
 	}
 	if (mountpoint_fd != -1)
-		close(mountpoint_fd);
+		fcntl(mountpoint_fd, F_SETFD, FD_CLOEXEC);
 
 	if (res == -1) {
 		close(fd);
+		if (mountpoint_fd != -1)
+			close(mountpoint_fd);
 		return -1;
 	}
 
 	if (geteuid() == 0) {
 		res = add_mount(source, mnt, type, mnt_opts);
 		if (res == -1) {
-			umount2(mnt, 2); /* lazy umount */
+			if (isdir && mountpoint_fd != -1) {
+				res = fchdir(mountpoint_fd);
+				if (res == -1) {
+					close(mountpoint_fd);
+					close(fd);
+					return -1;
+				}
+			}
+			umount2(real_mnt, 2); /* lazy umount */
+			if (mountpoint_fd != -1)
+				close(mountpoint_fd);
 			close(fd);
 			return -1;
 		}
 	}
 
+	if (mountpoint_fd != -1)
+		close(mountpoint_fd);
+
 	free(source);
 	free(type);
 	free(mnt_opts);
